/*-
 * Copyright (c) 2014, ServerU Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/errno.h>
#include <sys/conf.h>
#include <sys/module.h>

#include <sys/time.h>
#include <sys/proc.h>
#include <sys/unistd.h>
#include <sys/fcntl.h>
#if __FreeBSD_version < 800000
#include <sys/ioctl.h>
#endif
#include <sys/sysctl.h>
#include <sys/signalvar.h>

#include <sys/uio.h>
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>

#include "bpwd_ioctl.h"
#include "ioaccess.h"
#include "version.h"


#define inportb(x) inb(x)
#define outportb(y,x) outb(y,x)
#define inportl(x) inl(x)
#define outportl(y,x) outl(y,x)

#define I2C_READ	0x01
#define I2C_WRITE	0x00
#define udelay(a) DELAY(a)

/*
 * Add for AMD
 */
#define PCI_INDEX_PORT	0xCF8
#define PCI_DATA_PORT	0xCFC
#define PCIADDR		0x80000000

#define PMIO_INDEX_PORT	0xCD6
#define PMIO_DATA_PORT	0xCD7
#define SmBus0En	0x2C

/* io base */
int io_base=0;

/* previous prototype */
int probe_smbus_controller(void) ;	

/* Function prototypes */
static d_open_t      bpwd_open;
static d_close_t     bpwd_close;
static d_ioctl_t     bpwd_ioctl;
//static d_read_t      echo_read;
//static d_write_t     echo_write;


static struct cdevsw bpwd_devsw = {
	.d_version = D_VERSION, /* version */	
	.d_open = bpwd_open,   /* open */	
	.d_close = bpwd_close, /* close */	
	.d_ioctl = bpwd_ioctl, /* ioctl */	
	.d_name = "bpwd"        /* name */
	//.d_read = mydev_read,   /* read */	
	//.d_write = mydev_write, /* write */		
};

/* 
 * Used as the variable that is the reference to our device
 * in devfs... we must keep this variable sane until we 
 * call kldunload.
 */
static struct cdev *bpwd_dev;


//probe smbus
int probe_smbus_controller(void) 
{
  unsigned long addr, data_tmp, amd_base;
	
	//check the chipset smbus controller is working?
	addr= (PCIADDR) | ((0x1F <<3)<<8) | (0x03<<8 | 0x08);
	outportl(PCI_INDEX_PORT, addr);
	data_tmp=inportl(PCI_DATA_PORT);
		
	
	if((data_tmp!=0xffffffff)&&( (data_tmp>>16)==(0x0c05)))
	{
		//try to get smbus base address
		addr=(PCIADDR) | ((0x1F <<3)<<8) | (0x03<<8)|0x20;
		outportl(PCI_INDEX_PORT, addr);
		data_tmp=inportl(PCI_DATA_PORT);
		return (data_tmp&0x0000fffe);	
	}
	
	else {
		return 0; 
    }
 //=================================================================       
  int err=0,ret_value=0;
  
	unsigned char tmp,ret_val;

	addr = (PCIADDR) | (0x14 << 3 <<8) | (0x00 << 8 | 0x00);
	outportl(PCI_INDEX_PORT,addr);
	data_tmp=inportl(PCI_DATA_PORT);
	printf("AMD Vendor ID: 0x%lx\n",data_tmp);
	if(data_tmp == 0x43851002)
	{
		printf("Match vendor ID !!\n");
		//Get AMD smbus base address
		outportb(PMIO_INDEX_PORT, SmBus0En+1);
		tmp=inportb(PMIO_DATA_PORT);
		amd_base = (amd_base) | tmp;
		outportb(PMIO_INDEX_PORT, SmBus0En+2);
		tmp=inportb(PMIO_DATA_PORT);
		amd_base = (amd_base << 4) | tmp;
		outportb(PMIO_INDEX_PORT, SmBus0En+3);
		tmp=inportb(PMIO_DATA_PORT);
		amd_base = (amd_base << 4) | tmp;
		//printk("Base address %d : %x\n",i,tmp);
		//printk("AMD Base Address: 0x%lx\n",amd_base);
		io_base = amd_base;
		ret_value = i2c_smbus_read_byte_data(io_base, 0x37, 0,&ret_val,1);
		if(ret_value == 0)
			return err;	
		//printk("Return Value : %d\n",ret_value);
		io_base = io_base | 0x20;
		ret_value = i2c_smbus_read_byte_data(io_base, 0x37, 0,&ret_val,1);
		if(ret_value == 0)
			return err;
		err= -ENODEV;
		return err;
		//printk("Return Value : %d\n",ret_value);
		//printk("---AMD Base Address: 0x%x\n",io_base);      
        
  //====================================================================            
            }                  
   }

/* return 0 mean host ready for transfer, otherwise return -1 */
int host_not_ready(int fd)
{
    unsigned char temp;
    int timeout = 0;
    outportb(fd,0xFF);		// Host Status Register (00)
    udelay(1000);			
    temp = inportb(fd);
    /* Make sure the SMBus host is ready to start transmitting */
    /* 0x1f = Failed, Bus_Err, Dev_Err, Intr, Host_Busy */
    if ((temp = (0x1f & inportb(fd + SMBHSTSTS))) != 0x00) {
        do {
            outportb(fd + SMBHSTSTS, temp);
            udelay(1000);
            temp = inportb(fd + SMBHSTSTS);
        } while (( (temp & 0x1f) != 0x00) && (timeout++ < MAX_TIMEOUT));
//        printf("temp = %x timeout = %x\n", temp, timeout);
        if ( timeout >= MAX_TIMEOUT) {
//            printf("Temp = %x\n", temp);
            return -1;
        }    
    }
    return 0;
}

/* wait command processed and check error occured */
int host_busy_or_err(int fd, int inhibit_error)
{
    unsigned char temp;
    int     timeout = 0;
    
    /* We will always wait for a fraction of a second! */
    do {
        udelay(1000);
        temp = inportb(fd + SMBHSTSTS);
    } while ((temp & SMBHSTSTS_HOST_BUSY) && (timeout++ < MAX_TIMEOUT));

    /* If the SMBus is still busy, we give up */
    if (timeout >= MAX_TIMEOUT) {
        //fprintf(stderr, "SMBus Timeout!\n");
		printf("SMBus Timeout!\n");
        outportb(fd+ SMBHSTCNT, inportb(fd + SMBHSTCNT) | SMBHSTCNT_KILL);
        udelay(1000);
        outportb(fd+ SMBHSTCNT, inportb(fd + SMBHSTCNT) & (~SMBHSTCNT_KILL));
        return -1;
    }
    
    if ( temp & (SMBHSTSTS_FAILED | SMBHSTSTS_BUS_ERR | SMBHSTSTS_DEV_ERR)) {
        if(inhibit_error != 1) /* do not show error message for scan mode */
            //fprintf(stderr, "SMBus Error\n");
			printf("SMBus Error\n");
        return -1;
    }   
     
    if ((inportb( fd + SMBHSTSTS) & 0x1f) != 0x00)
        outportb(fd + SMBHSTSTS, inportb(fd+SMBHSTSTS));

    
    return 0;
}

//i2c write byte
int i2c_smbus_write_byte(int fd, unsigned char i2c_address,
                        unsigned char i2c_command,
                        int pec, unsigned char *pec_byte)
{
    
    if ( host_not_ready(fd) ) return -1;
    
    	outportb(fd + SMBHSTCMD, i2c_command);        // Host Command Register (03)
	outportb(fd + SMBHSTADD, (i2c_address<<1|I2C_WRITE)); //Host Address Register (04)
	if ( pec ) {
        	outportb(fd + SMBPEC, *pec_byte);      // PEC Register (08)
        	outportb(fd + SMBHSTCNT, 0x44 | 0x80);    // Host Control Register (02)
                                        // Write byte with PEC and Start
    	} else
        	outportb(fd + SMBHSTCNT, 0x44); // Host Control Register (02)
                                        // Write byte protocol and Start
		
    if ( host_busy_or_err(fd, 0)) 
        return -1;
    else
        return 0;		                    
} 
//i2c write byte data
int i2c_smbus_write_byte_data(int fd, unsigned char i2c_address, 
                                unsigned char i2c_register, unsigned char value)
{
    
    if ( host_not_ready(fd) ) return -1;
							
	outportb(fd + SMBHSTCMD, i2c_register);	// Host Command Register (03)
	outportb(fd + SMBHSTADD,(i2c_address<<1|I2C_WRITE)); //Host Address Register (04)
	outportb(fd + SMBHSTDAT0, value);	   // Host Data 0 Register (05)
	outportb(fd + SMBHSTCNT, 0x48);        // Host Control Register (02)
                                           // Read byte protocol and Start
    if ( host_busy_or_err(fd, 0)) 
        return -1;
    else
        return 0;		                    
}

//i2c read byte data 
int i2c_smbus_read_byte_data(int fd, unsigned char i2c_address, 
                             unsigned char i2c_register, unsigned char *ret_val, int inhibit_error)
{
    if ( host_not_ready(fd) ) return -1;
						
	outportb(fd + SMBHSTCMD, i2c_register);  // Host Command Register (03)
	outportb(fd + SMBHSTADD,(i2c_address<<1|I2C_READ)); //Host Address Register (04)	 // Set the I2C address
	outportb(fd + SMBHSTCNT, 0x48);          // Host Control Register (02)
				                             // Read byte protocol and Start
											 
    if ( host_busy_or_err(fd, inhibit_error)) return -1;
	
    *ret_val = inportb(fd + SMBHSTDAT0);
	return 0;  //read OK
}
	
//i2c read word data	
int i2c_smbus_read_word_data(int fd, unsigned char i2c_address, 
                            unsigned char i2c_register, unsigned short *ret_val)
{
    if ( host_not_ready(fd) ) return -1;
    					
	outportb(fd + SMBHSTCMD, i2c_register);  // Host Command Register (03)
	outportb(fd + SMBHSTADD,(i2c_address<<1|I2C_READ)); //Host Address Register (04)	 // Set the I2C address
	outportb(fd + SMBHSTCNT, 0x4C);          // Host Control Register (02)
				                             // Read word protocol and Start

    if ( host_busy_or_err(fd, 0)) return -1;
    
    *ret_val = ((inportb(fd + SMBHSTDAT1) << 8) | inportb(fd + SMBHSTDAT0));
	   
	return 0;  //read OK
}




int
bpwd_open(struct cdev *dev, int flag, int otyp, struct thread *td)
{
    //uprintf("BPWD device open.\n");
    return (0);
}

int
bpwd_close(struct cdev *dev, int flag, int otyp, struct thread *td)
{
    //uprintf("BPWD device close.\n");
    return (0);
}
int
bpwd_ioctl(struct cdev *dev, u_long cmd, caddr_t data, int mode,
    struct thread *td)
{
    int error = 0;
    bp_s *bp=(bp_s *)data;
    switch(cmd) 
	{
		case IOCTL_READ_BYTE_DATA_CMD:
			if(i2c_smbus_read_byte_data(io_base,bp->bp_addr,bp->bp_cmd,&bp->bp_buffer[0],bp->bp_inhibit_err))
				return -EFAULT;
			else
			{
				return 0;
			}
			break;
		case IOCTL_READ_WORD_DATA_CMD:
			if(i2c_smbus_read_word_data(io_base,bp->bp_addr,bp->bp_cmd,(unsigned short *)&bp->bp_buffer[0]))
				return -EFAULT;
			else
			{
				return 0;
			}
			break;
		case IOCTL_WRITE_BYTE_CMD:
			if(i2c_smbus_write_byte(io_base, bp->bp_addr, bp->bp_cmd, 0,&bp->bp_pec))
				return -EFAULT;
			else
			{
				return 0;
			}
			break;
		case IOCTL_WRITE_BYTE_DATA_CMD:
			if(i2c_smbus_write_byte_data(io_base,bp->bp_addr,bp->bp_cmd,bp->bp_buffer[0]))
				return -EFAULT;
			else
			{
				return 0;
			}
			break;
		default:
			//printf("No such ioctl for me!\n");
			error = EINVAL;
			break;
    }
    return (error);
}

/*
 * This function is called by the kld[un]load(2) system calls to
 * determine what actions to take when a module is loaded or unloaded.
 */
static int
bpwd_loader(struct module *m, int what, void *arg)
{
    int err = 0;

    switch (what) {
		case MOD_LOAD:                /* kldload */
			bpwd_dev = make_dev(&bpwd_devsw, 0, UID_ROOT, GID_WHEEL, 0600, "bpwd");
			io_base=probe_smbus_controller();
			uprintf("ServerU Bypass Watch Dog (BPWD) device loaded.\n");
			break;
		case MOD_UNLOAD:
			destroy_dev(bpwd_dev);
			uprintf("ServerU Bypass Watch Dog (BPWD) device unloaded.\n");
			break;
		default:
			err = EOPNOTSUPP;
			break;
    }
    return(err);
}


DEV_MODULE(bpwd,bpwd_loader,NULL);
