/*-
 * Copyright (c) 2014, ServerU Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#include <sys/types.h>
#include <sys/ioctl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <err.h>

#include "bpwd_ioctl.h"
#include "version.h"
#include "bp_def.h"

int i2c_smbus_read_byte_data(int fd, unsigned char i2c_address, 
                             unsigned char i2c_register, unsigned char *ret_val, int inhibit_err);
int i2c_smbus_read_word_data(int fd, unsigned char i2c_address, 
                                      unsigned char i2c_register, unsigned short *ret_val);
int i2c_smbus_write_byte_data(int fd, unsigned char i2c_address, 
                                unsigned char i2c_register, unsigned char value);
int i2c_smbus_write_byte(int fd, unsigned char i2c_address,
                        unsigned char i2c_command,
                        int pec, unsigned char *pec_byte);		
/* show help */
void print_help(void);
/* Scan i2c bus and print out device */
void scan_i2c_bus( int fd );
/* Get information */
int get_information( int fd );
/* Get i2c bus */
/*
 * adapter_number ==> smbus adapter number, enumerate by i2c subsystem
 */ 
int adapter_number=0;

/*
 * scan_mode=0 ==> SMBUS read/write 
 * scan_mode=1 ==> Try to scan SMBUS
 * scan_mode=2 ==> Try to do ARP
 * scan_mode=3 ==> Show Bypass module information
 * scan_mode=4 ==> Send command I2C switch to get throug bus operation
 */ 
int scan_mode=-1;

/*
 * inhibit_error=1 ==> Do not report device error
 */ 
int inhibit_error=0;

/* 
 * rw_mode=0 ==> write
 * rw_mode=1 ==> read
 */
int rw_mode;

/* 
 * byte_word=0 ==> byte
 * byte_word=1 ==> word
 */
int byte_word;

/*
 * motherboard=0 ==> no i2c switch
 * motherboard=1 ==> with i2c switch
 */
int motherboard=0;

 
int verbose = 0;

/*
 * model_name
 */ 
char model_name[16];

char *program_name;

/* 
 * slave_address ==> 7-bit address
 */ 
unsigned int slave_address = 0x00;
unsigned int reg_no = 0xffff;
unsigned int write_data;

int adapter_no;



typedef struct {
    char model[16];
    unsigned char switch_addr;
    unsigned char switch_data;
} model_method_t;

#define MODEL_GET_THROUGH_NUM	5
model_method_t model_get_through_tbl[MODEL_GET_THROUGH_NUM+1]= {
	//{"887X", 0x73, 0x0F},
	{"887X"   , 0x73, 0x00},
	{"9655"   , 0x70, 0x00},
	{"8895"   , 0x72, 0x00},
	{"8865EXT", 0x73, 0x00},
	{"7582EXT", 0x73, 0x00},
	{0, 0, 0}
};

typedef struct {
    unsigned char cmd;
    char cmd_desc[50];
} cmd_t;

cmd_t info_tbl[]={
    {MAJOR_VER_CMD      ,"FW major version: "},                     
    {MINOR_VER_CMD      ,"FW minor version: "},
    {MODULE_CAP_CMD     ,"Module capability: \n"},                     
    {SO_BP_CAP_CMD      ,"System-Off Bypass capability: Support "},                     
    {JO_BP_CAP_CMD      ,"Just-On Bypass capability: Support "},                     
    {RT_BP_CAP_CMD      ,"Run-Time Bypass capability: Support "},                     
    {RT_WDT1_CAP_CMD    ,"Run-Time watchdog1 timer capability: 1~"},                     
    {RT_WDT2_CAP_CMD    ,"Run-Time watchdog2 timer capability: 1~"},                     
    {JO_WDT3_CAP_CMD    ,"Just-On watchdog3 timer capability: 5~"},                     
    {SO_BP_SET_CMD      ,"System-Off Bypass setting:\n"},                    
    {JO_BP_SET_CMD      ,"Just-On Bypass setting:\n"},                     
    {RT_BP_SET_CMD      ,"Run-Time Bypass setting:\n"},                     
    {RT_WDT1_STS_CMD    ,"Run-Time watchdog1 timer status: "},                     
    {RT_WDT1_PAIR_CMD   ,"Run-Time watchdog1 pair setting:\n"},                     
    {RT_WDT1_TIMER_CMD  ,"Run-Time watchdog1 timer count: "},                     
    {RT_WDT2_STS_CMD    ,"Run-Time watchdog2 timer status: "},                     
    {RT_WDT2_PAIR_CMD   ,"Run-Time watchdog2 pair setting:\n"},                     
    {RT_WDT2_TIMER_CMD  ,"Run-Time watchdog2 timer count: "},                     
    {JO_WDT3_STS_CMD    ,"Just-On watchdog3 timer status: "},                     
    {JO_WDT3_PAIR_CMD   ,"Just-On watchdog3 pair setting:\n"},                     
    {JO_WDT3_TIMER_CMD  ,"Just-On watchdog3 timer count: "},
    {0, 0},                     
}; 

int main( int argc, char** argv )
{
	int fd,error;
	int i, j, len;
	char str[20];
	//CRC_TYPE tmp;
	unsigned char pec_byte;
	unsigned char ret_val_8;
	unsigned short ret_val_16;
    unsigned char ret_array[50];
	//struct bp_s bp;	

    if ( verbose ) {
	   fprintf(stdout, "=== ServerU Inc.===\n");;
	   fprintf(stdout, "Bypass module access utility "CODE_VERSION"\n\n");;
	}   

	if ( parse_argument(argc, argv)) {
		print_help();
		return -1;
	}
	fd = open("/dev/bpwd",O_RDWR);
	if (fd == -1)
		err(1, "Failed to probe SMBUS controller\n");

	if (scan_mode == 1) {
        // Try to scan i2c bus 
		scan_i2c_bus(fd);
		return 0;
	}/* 
    if (scan_mode == 2) { 
        // Try to do SMBUS 2.0 ARP 
        return arp_i2c_bus(fd);
    }*/
    if (scan_mode == 3) {
        // Try to do SMBUS 2.0 ARP 
        return get_information(fd);
    }
    if (scan_mode == 4) {
        // Send command to I2C switch to get through bus operation 
        return get_through_i2c_bus(fd,slave_address);
    }
	// i2c read 
    if (rw_mode == 1) {
		if(motherboard==1)
		{
			get_through_i2c_bus(fd,slave_address);
			slave_address=0x30;
		}
	    fprintf(stdout,"READ ADDRESS:0x%2x Command:0x%2x ...", slave_address, reg_no);
		if (byte_word == 0 ) {        
			
		    if (i2c_smbus_read_byte_data(fd, slave_address, reg_no, &ret_val_8, 0))  
			//if(ioctl(fd,IOCTL_READ_BYTE_DATA_CMD))
                fprintf(stdout, "FAIL\n");
            else 
                fprintf(stdout, "OK, DATA= 0x%2.2x\n", ret_val_8);
		}    
		if (byte_word == 1 ) {        
		    if (i2c_smbus_read_word_data(fd, slave_address, reg_no, &ret_val_16)) 
				fprintf(stdout, "FAIL\n");
			else 
				fprintf(stdout, "OK, DATA = 0x%4.4x\n", ret_val_16);
		}    
	}
	// i2c write 
	if ((rw_mode == 0) && (slave_address != 0x50)) {
		if(motherboard==1)
		{
			get_through_i2c_bus(fd,slave_address);
			slave_address=0x30;
		}
		//slave_address=0x30;
		fprintf(stdout,"WRITE ADDRESS:0x%2x  Command:0x%2x  Data:0x%2x...", slave_address, reg_no, write_data); 
		if (i2c_smbus_write_byte_data(fd, slave_address, reg_no, write_data))
          fprintf(stdout, "FAIL\n");
        else
          fprintf(stdout, "OK\n");
    }
	close(fd);
	exit(0);
}
/*
 * Parse argument, if error, return -1
 */ 
int parse_argument(int argc, char** argv)
{
	int write_data_available=0;
	int ret_val=0;
	int opt;
	int argument_available = 0;

    program_name = argv[0];
    
    while ((opt = getopt(argc, argv, "+SIvrwh:M:d:c:o:a:")) != -1) {
        argument_available = 1;
        switch (opt) {
            case 'S':
                scan_mode = 1;
                inhibit_error = 1;
                break;
            case 'M':
                sscanf(optarg, "%s", model_name);
				//scan_mode = 4;
				motherboard=1;
                break;
            case 'I':
                scan_mode = 3;
                break;
            case 'r':
                scan_mode = 0;
                rw_mode = 1;
                byte_word =1;
                break;    
            case 'w':
                scan_mode = 0;
                rw_mode = 0;
                byte_word =0;
                break;    
            case 'd':
                sscanf(optarg, "%x", &slave_address);
                break;    
            case 'c':
                sscanf(optarg, "%x", &reg_no);
                break;    
            case 'o':
                sscanf(optarg, "%x", &write_data);
                write_data_available=1;
                break;
            case 'a':
                sscanf(optarg, "%d", &adapter_number);
                break;    
            case 'v':
                verbose = 1;
                break;    
            case 'h':
                return -1;
                break;    
            default:
                fprintf(stderr, "Unknown argument \"%c\"\n", opt);
                ret_val = -1;
        }
    }
//debug +
//	printf("scan_mode=%x rw_mode=%x byte_word=%x slave_address=%x command=%x write_data=%x\n", 
//		scan_mode, rw_mode, byte_word, slave_address, reg_no, write_data);
//debug -
    if (!argument_available) return -1;
    if ( scan_mode == -1 ) {
        ret_val=-1;
        fprintf(stderr, "Error: Unknown operation to do, try -r | -w\n");
    }
    if ( (scan_mode==0) && (rw_mode==0) && (write_data_available==0)) {
        ret_val = -1;
        fprintf(stderr, "Error: No output data for write function, try -o [byte_data]\n");
	}
	if (((scan_mode==0) && (slave_address == 0x00)) || 
        ((scan_mode==3) && (slave_address == 0x00))){
        ret_val=-1;
        fprintf(stderr, "Error: Unknown device address, try -d [i2c_address]\n");
	}
	if ( (scan_mode == 0) && (reg_no == 0xffff)) {
        ret_val=-1;
        fprintf(stderr, "Error: Unknown command, try -c [command]\n");
	}
	return ret_val;
       
}

/*
 * Print Help
 */
void print_help(void)
{
	int i; 
    	model_method_t *model_ptr;
	
    	model_ptr = model_get_through_tbl;

	fprintf(stdout,"ServerU Bypass/Watchdog Access Code: " CODE_VERSION"\n");;

	fprintf(stdout,
        "Usage: %s -S\n"\
		"\tScan known device.\n"\
		"\n", program_name);
	fprintf(stdout,
		"Usage: %s -r -d [i2c_address] -c [command]\n"\
		"\tSMBUS read word data protocol\n"\
        "\n", program_name);
	fprintf(stdout,	
		"Usage: %s -w -d [i2c_address] -c [command] -o [byte_data]\n"\
		"\tSMBUS write byte data protocol\n"\
	"\n", program_name);
	fprintf(stdout,	
		"Usage: %s -I -d [i2c_address]\n"\
		"\tGet Bypass module information\n\n", program_name);
	fprintf(stdout,
        "Global argument:\n"\
        "\t-v: be verbose\n"\
        "\t-h: help\n"); 
        /* XXXSU Patrick removed manual mode and we are using only default (0x37) 
	* fprintf(stdout, 
	* 	"Support %d model_name:\n", MODEL_GET_THROUGH_NUM);
	* for(i=0; i<MODEL_GET_THROUGH_NUM; i++, model_ptr++) {
	*    fprintf(stdout, 
	*	    "\t %s \n",model_ptr->model);
	* }
	*/

}
/*
 * Scan I2C Bus
 */
void scan_i2c_bus( int fd )
{
	int i;
	unsigned char ret_val;
	unsigned char i2c_ret_val;
	unsigned char scan_addr;
/* 
 * Almost all device support read byte or read byte data command
 * Try to probe known device
 */ 	
	unsigned char known_address[]={	
//      0x2C, //W83792D HW monitor
//      0x2D, //W83792D HW monitor
//      0x2E, //W83792D HW monitor
//      0x2F, //W83792D HW monitor
        0x30, //Bypass Module
        0x31, //Bypass Module
        0x32, //Bypass Module
        0x33, //Bypass Module
        0x34, //Bypass Module
        0x35, //Bypass Module
        0x36, //Bypass Module
        0x37, //Bypass Module (ServerU default)
//      0x50, //EEPROM
//      0x51, //EEPROM
//      0x52, //EEPROM
//      0x53, //EEPROM
//      0x6A  //clock gen
	};
	if(motherboard==1)  //with i2c switch
	{
		model_method_t *model_ptr;
		model_ptr=model_get_through_tbl;
		while(model_ptr->switch_addr!=0)
		{
			if(!strcmp(model_name,model_ptr->model))
			{
				if(strcmp(model_name,"887X")==0)
				{
					//bp.bp_addr=model_ptr->switch_addr;
					if(!(i2c_smbus_read_byte_data(fd,model_ptr->switch_addr,0,&i2c_ret_val,1)))
					{
						if(i2c_ret_val==1)
						{
							i2c_ret_val=48;
							model_ptr->switch_data=0x01;
						}
						if(i2c_ret_val==2)
						{
							i2c_ret_val=49;
							model_ptr->switch_data=0x02;
						}	
						if(i2c_ret_val==4)
						{
							i2c_ret_val=50;
							model_ptr->switch_data=0x04;
						}
						if(i2c_ret_val==8)
						{
							i2c_ret_val=51;
							model_ptr->switch_data=0x08;
						}
					}
					i2c_smbus_write_byte(fd,model_ptr->switch_addr,model_ptr->switch_data,0,NULL);
				}
				if(strcmp(model_name,"9655")==0)
				{
					if(!i2c_smbus_read_byte_data(fd,model_ptr->switch_addr,0,&i2c_ret_val,1))
					{
						if(i2c_ret_val==32)
						{
							i2c_ret_val=48;
							model_ptr->switch_data=0x20;
						}
						if(i2c_ret_val==64)
						{
							i2c_ret_val=49;
							model_ptr->switch_data=0x40;
						}
						if(i2c_ret_val==128)
						{
							i2c_ret_val=50;
							model_ptr->switch_data=0x80;
						}
					}
					i2c_smbus_write_byte(fd,model_ptr->switch_addr,model_ptr->switch_data,0,NULL);
				}
			}
			model_ptr++;
		}
		for(i=0;i<sizeof(known_address);i++)
		{
			if(known_address[i]==48 || known_address[i]==49 || known_address[i]==50 || known_address[i]==51)
			{
				if(known_address[i]==i2c_ret_val)
				{
            				fprintf(stdout, "Address:0x%x response with read byte command, value = 0x%x\n",known_address[i], i2c_ret_val);
				}
				else
				{
            				fprintf(stdout, "Address:0x%x No response\n", known_address[i]);
				}
			}
			else
			{
				if(i2c_smbus_read_byte_data(fd, known_address[i], 0, &ret_val, 1)) 			  		     {
            				fprintf(stdout, "Address:0x%x No response\n", known_address[i]);
				}
				else
				{
            				fprintf(stdout, "Address:0x%x response with read byte command, value = 0x%x\n",known_address[i], ret_val);
				}
			}
		}
	}
	else //no i2c switch
	{
		for ( i = 0; i < sizeof(known_address); i++)
		{
            scan_addr = known_address[i];
            if ( (scan_addr <= 0x37) && (scan_addr >=0x30)) {
			if (motherboard == 1) {
				if ( get_through_i2c_bus(fd, scan_addr, 0)) return;
				scan_addr = 0x30;
			    }
	      	}
            
			if(i2c_smbus_read_byte_data(fd, known_address[i], 0, &ret_val, 1)) 			  			 {
            			fprintf(stdout, "Slot %d (Address:0x%x) No response\n",i+1, known_address[i]);
			}
			else 
			{
            			fprintf(stdout, "Slot %d (Address:0x%x) response with read byte command, value = 0x%x\n",i+1, known_address[i], ret_val);
  			}
		}
  	}
	return;
}

int get_information( int fd )
{
 	int i;
	unsigned short ret_val_16;
	unsigned char ret_val, temp;
	cmd_t *info_tbl_ptr;
	unsigned char so_bp_cap, jo_bp_cap, rt_bp_cap;
	//slave_address=0x30;
	if(motherboard==1)
	{
        if (get_through_i2c_bus(fd,slave_address, 1)) return -1;              
		slave_address=0x30;
	}
	else
	{
		slave_address=0x37;
	}
	info_tbl_ptr = info_tbl;
	while ( info_tbl_ptr->cmd != 0 ) {
        if (i2c_smbus_read_word_data(fd, slave_address, info_tbl_ptr->cmd, &ret_val_16)) { 
            fprintf(stdout, "Command Fail, check -d [i2c_address} is correct\n");
            break;
        }
        if ( (ret_val_16 & 0x00ff) != (info_tbl_ptr->cmd | CMD_SUCCESS)) { 
            fprintf(stdout, "Un-support Command, check specification\n");
        } else {
            ret_val = (ret_val_16 >> 8);    
            fprintf(stdout, "(cmd 0x%2.2x)%s", info_tbl_ptr->cmd, info_tbl_ptr->cmd_desc);
            switch ( info_tbl_ptr->cmd ) {
                case MAJOR_VER_CMD:                     
                case MINOR_VER_CMD:
                    fprintf(stdout, "%d\n", (ret_val));
                    break;
                case MODULE_CAP_CMD:
                    fprintf(stdout, "\tSystem-Off bypass ");                     
                    if ( !(ret_val & MODULE_CAP_SO_BIT)) 
                        fprintf(stdout, "Not ");
                    fprintf(stdout, "Supported\n");
                                             
                    fprintf(stdout, "\tJust-On bypass ");                     
                    if ( !(ret_val & MODULE_CAP_JO_BIT)) 
                        fprintf(stdout, "Not ");
                    fprintf(stdout, "Supported\n");
                                             
                    fprintf(stdout, "\tRun-Time bypass ");                     
                    if ( !(ret_val & MODULE_CAP_RT_BIT)) 
                        fprintf(stdout, "Not ");
                    fprintf(stdout, "Supported\n");
                                             
                    fprintf(stdout, "\tRun-Time Watchdog1 timer ");                     
                    if ( !(ret_val & MODULE_CAP_WDT1_BIT)) 
                        fprintf(stdout, "Not ");
                    fprintf(stdout, "Supported\n");
                                             
                    fprintf(stdout, "\tRun-Time Watchdog2 timer ");                     
                    if ( !(ret_val & MODULE_CAP_WDT2_BIT)) 
                        fprintf(stdout, "Not ");
                    fprintf(stdout, "Supported\n");      
                                       
                    fprintf(stdout, "\tJust-On Watchdog3 timer ");                     
                    if ( !(ret_val & MODULE_CAP_WDT3_BIT)) 
                        fprintf(stdout, "Not ");
                    fprintf(stdout, "Supported\n"); 
                    break;                        
                case SO_BP_CAP_CMD:              
                    so_bp_cap = ret_val;
                    i=0;       
                    while(ret_val&0x01) {i++; ret_val>>=1;}                           
                    fprintf(stdout, "%d bypass pairs\n", i);                     
                    break;                        
                case JO_BP_CAP_CMD:                           
                    jo_bp_cap = ret_val;
                    i=0;       
                    while(ret_val&0x01) {i++; ret_val>>=1;}                           
                    fprintf(stdout, "%d bypass pairs\n", i);                     
                    break;                        
                case RT_BP_CAP_CMD:
                    rt_bp_cap = ret_val;       
                    i=0;
                    while(ret_val&0x01) {i++; ret_val>>=1;}                           
                    fprintf(stdout, "%d bypass pairs\n", i);                     
                    break;                        
                case RT_WDT1_CAP_CMD:                         
                case RT_WDT2_CAP_CMD:
                    fprintf(stdout, "%d seconds\n", (int)ret_val);
                    break;                     
                case JO_WDT3_CAP_CMD:                     
                    fprintf(stdout, "%d seconds ( 1 scale = 5 seconds)\n", (int)ret_val*5);
                    break;                     
                case SO_BP_SET_CMD:                    
                    i=1;
                    temp = so_bp_cap;
                    while (temp & 0x01 ) {
                        fprintf(stdout,"\tPair %d bypass ", i);
                        if (ret_val & 0x01)
                            fprintf(stdout,"Enable\n");
                        else    
                            fprintf(stdout,"Disable\n");
                        temp >>=1;
                        ret_val >>=1;
                        i++;
                    }
                    break;
                case JO_BP_SET_CMD:                     
                    i=1;
                    temp = jo_bp_cap;
                    while (temp & 0x01 ) {
                        fprintf(stdout,"\tPair %d bypass ", i);
                        if (ret_val & 0x01)
                            fprintf(stdout,"Enable\n");
                        else    
                            fprintf(stdout,"Disable\n");
                        temp >>=1;
                        ret_val >>=1;
                        i++;
                    }
                    break;
                case RT_BP_SET_CMD:
                    i=1;
                    temp = rt_bp_cap;
                    while (temp & 0x01 ) {
                        fprintf(stdout,"\tPair %d bypass ", i);
                        if (ret_val & 0x01)
                            fprintf(stdout,"Enable\n");
                        else    
                            fprintf(stdout,"Disable\n");
                        temp >>=1;
                        ret_val >>=1;
                        i++;
                    }
                    break;
                case RT_WDT1_STS_CMD:                     
                case RT_WDT2_STS_CMD:                     
                case JO_WDT3_STS_CMD:
                    switch (ret_val) {
                        case WDT_STS_STOP:
                            fprintf(stdout, "Timer Stop\n");
                            break;
                        case WDT_STS_RUNNING:
                            fprintf(stdout, "Timer Running\n");
                            break;
                        case WDT_STS_EXPIRED:
                            fprintf(stdout, "Timer Expired\n");
                            break;
                        default:
                            fprintf(stdout, "Unknown state\n");
                            break;
                    }
                    break;                 
                case RT_WDT1_PAIR_CMD:                     
                case RT_WDT2_PAIR_CMD:                     
                case JO_WDT3_PAIR_CMD:
                    i=1;
                    if ( ret_val == 0) {
                        fprintf(stdout,"\tNo Pair setting for timeout\n");
                    } else {
                        temp = rt_bp_cap;
                        while (temp & 0x01 ) {
                            if (ret_val & 0x01)
                                fprintf(stdout,"\tPair %d bypass will Enable while timeout\n", i);
                            temp >>=1;
                            ret_val >>=1;
                            i++;
                        }
                    }    
                    break;
                                     
                case RT_WDT1_TIMER_CMD:                     
                case RT_WDT2_TIMER_CMD:                     
                    fprintf(stdout, " %d seconds\n", (int)ret_val);
                    break;                     
                case JO_WDT3_TIMER_CMD:
                    fprintf(stdout, " %d seconds ( 1 scale = 5 seconds)\n", (int)ret_val*5);
                    break;                     
            }
        }       
        info_tbl_ptr++;   
    }
    /* Get board ID */
    /* first, write byte data to reset board id pointer, 
       then read word data 3 times sequential*/
    if (i2c_smbus_write_byte_data(fd, slave_address, BOARD_ID_CMD, 0)) {
        return -1;
    }
    fprintf(stdout, "(cmd 0x%2.2x)Get Board ID:", BOARD_ID_CMD);
    for ( i=0; i<3; i++) {
        if (i2c_smbus_read_word_data(fd, slave_address, BOARD_ID_CMD, &ret_val_16)) { 
            fprintf(stdout, "\nCommand Fail, check -d [i2c_address} is correct\n");
            return -1;
        }
        fprintf(stdout, "%2.2x %2.2x ", (ret_val_16&0x00ff), (ret_val_16>>8));
    }
    fprintf(stdout, "\n");    
    return 0;
}

/*
 * Get i2c bus
 */
int get_through_i2c_bus( int fd ,int slave_addr, int v)
{

    model_method_t *model_ptr;
    unsigned char second_layer_addr;
	
    model_ptr = model_get_through_tbl;
    while ( model_ptr->switch_addr != 0 ) {
		if(!strcmp(model_name, model_ptr->model)) {
                               
                               
             
             
             
             
             if(strcmp(model_ptr->model,"7582EXT")==0)  //PCA9545
			{
				if((slave_addr==0x30)||(slave_addr==0x31)||(slave_addr==0x32))
				{
				    /* Slot 1~3 is located at bottom module, left to right */
				    if (slave_addr == 0x30) {
				    	model_ptr->switch_data=0x01;  //channel-00
				    } else if (slave_addr == 0x31) {
				    	model_ptr->switch_data=0x02;  //channel-01
				    } else {
				    	model_ptr->switch_data=0x04;  //channel-02
				    }
				    if(i2c_smbus_write_byte(fd, model_ptr->switch_addr, model_ptr->switch_data, 0, NULL)) {
			            	if (v)
				        	fprintf(stderr, "Get through first I2C BUS FAILED\n");
				    		return -1;
			        	}  
		    			if (v)
				    		fprintf(stdout, "Get through first I2C Bus successful\n");
					return 0;
				}
				else if((slave_addr==0x33)||(slave_addr==0x34)||(slave_addr==0x35))
				{
				    /* Slot 4~6 is located at top module, left to right */
					model_ptr->switch_data=0x08;  //channel-03
				}
				else
				{
				    if (v) {
						fprintf(stderr, "7582EXT wrong Slave Address\n");
					}
					return -1;
				}
				if(i2c_smbus_write_byte(fd, model_ptr->switch_addr, model_ptr->switch_data, 0, NULL)) {
			            if (v)
				        fprintf(stderr, "Get through first I2C BUS FAILED\n");
				    return -1;
			        }  
		    		if (v)
				    fprintf(stdout, "Get through first I2C Bus successful\n");
				    
				second_layer_addr=0x72;  //PCA9545 address
				if (slave_addr==0x33)
				{
				    /* Slot 4 is located at top module, left */
					model_ptr->switch_data=0x04;  //channel-02
				}
				else if(slave_addr==0x34)
				{
				    /* Slot 5 is located at top module, middle */
					model_ptr->switch_data=0x02;  //channel-01
				}
				else if(slave_addr==0x35)
				{
				    /* Slot 6 is located at top module, right */
					model_ptr->switch_data=0x01;  //channel-00
				}
				else
				{
				    if (v) {
						fprintf(stderr, "7582EXT wrong Slave Address\n");
					}
					return -1;
				}
			    	if(i2c_smbus_write_byte(fd, second_layer_addr, model_ptr->switch_data, 0, NULL)) {
			            if (v)
				        fprintf(stderr, "Get through second I2C BUS FAILED\n");
				    return -1;
			    	}
		    		if (v)
				    fprintf(stdout, "Get through second I2C Bus successful\n");
				return 0;
				
			} else
//================================================                  
            if(strcmp(model_ptr->model,"8865EXT")==0)  //PCA9545
			{
				if((slave_addr==0x30)||(slave_addr==0x31)||(slave_addr==0x32))
				{
				    /* Slot 1~3 is located at bottom module, left to right */
					model_ptr->switch_data=0x02;  //channel-02
				}
				else if((slave_addr==0x33)||(slave_addr==0x34)||(slave_addr==0x35))
				{
				    /* Slot 4~6 is located at top module, left to right */
					model_ptr->switch_data=0x01;  //channel-01
				}
				else
				{
				    if (v) {
						fprintf(stderr, "8865EXT wrong Slave Address\n");
					}
					return -1;
				}
				if(i2c_smbus_write_byte(fd, model_ptr->switch_addr, model_ptr->switch_data, 0, NULL)) {
			            if (v)
				        fprintf(stderr, "Get through first I2C BUS FAILED\n");
				    return -1;
			        }  
		    		if (v)
				    fprintf(stdout, "Get through first I2C Bus successful\n");
				    
				second_layer_addr=0x71;  //PCA9545 address
				if(slave_addr==0x30)
				{
				    /* Slot 1 is located at bottom module, left */
					model_ptr->switch_data=0x02;  //channel-02
//					second_layer_addr=0x73;  //PCA9545 address
				}
				else if(slave_addr==0x31)
				{
				    /* Slot 2 is located at bottom module, middle */
					model_ptr->switch_data=0x01;  //channel-01
//					second_layer_addr=0x73;  //PCA9545 address
				}
				else if(slave_addr==0x32)
				{
				    /* Slot 3 is located at bottom module, right */
					model_ptr->switch_data=0x04;  //channel-03
//					second_layer_addr=0x73;  //PCA9545 address
				}
				else if(slave_addr==0x33)
				{
				    /* Slot 4 is located at top module, left */
					model_ptr->switch_data=0x01;  //channel-01
				}
				else if(slave_addr==0x34)
				{
				    /* Slot 5 is located at top module, middle */
					model_ptr->switch_data=0x02;  //channel-02
				}
				else if(slave_addr==0x35)
				{
				    /* Slot 6 is located at top module, right */
					model_ptr->switch_data=0x04;  //channel-03
				}
				else
				{
				    if (v) {
						fprintf(stderr, "8865EXT wrong Slave Address\n");
					}
					return -1;
				}
			    	if(i2c_smbus_write_byte(fd, second_layer_addr, model_ptr->switch_data, 0, NULL)) {
			            if (v)
				        fprintf(stderr, "Get through second I2C BUS FAILED\n");
				    return -1;
			    	}
		    		if (v)
				    fprintf(stdout, "Get through second I2C Bus successful\n");
				return 0;
				
			}
            
            
            
            
            
                                                                             
                               
                               
		    else if(strcmp(model_ptr->model,"887X")==0)  //PCA9545
			{
				if(slave_addr==48)  //0x30
				{
					model_ptr->switch_data=0x01;  //channel-01
				}
				else if(slave_addr==49)  //0x31
				{
					model_ptr->switch_data=0x02;  //channel-02
				}
				else if(slave_addr==50)  //0x32
				{
					model_ptr->switch_data=0x04;  //channel-03
				}
				else if(slave_addr==51)  //0x33
				{
					model_ptr->switch_data=0x08;  //channel-04
				}
				else
				{
                    if (v) {
					fprintf(stderr, "887X wrong Slave Address\n");
			 	    }
				    return -1;
			    }
            }
			else if(strcmp(model_ptr->model,"9655")==0)
			{
				//model_ptr->switch_data=0x20;
				if(slave_addr==48)  //0x30
				{
					model_ptr->switch_data=0x20;  //channel05
				}
				else if(slave_addr==49)  //0x31
				{
					model_ptr->switch_data=0x40;  //channel-06
				}
				else if(slave_addr==50)  //0x32
				{
					model_ptr->switch_data=0x80;  //channel-07
				}
				else
				{
                    
                    if (v) {
					fprintf(stderr, "9655 wrong Slave Address\n");
                   }
				     return -1;
                }    
			}
															
			else if(strcmp(model_ptr->model,"8895")==0)  //PCA9548(1111-0100)
			{
				if(slave_addr==48)  //0x30
				{
					model_ptr->switch_data=0x01;  //channel01
				}
				else if(slave_addr==49)  //0x31
				{
					model_ptr->switch_data=0x02;  //channel02
				}
				else if(slave_addr==50)  //0x32
				{
					model_ptr->switch_data=0x04;  //channel03
				}
				else if(slave_addr==51)  //0x33
				{
					model_ptr->switch_data=0x08;  //channel04
				}
				else if(slave_addr==52)  //0x34
				{
					model_ptr->switch_data=0x10;  //channel05
				}
				else if(slave_addr==53)  //0x35
				{
					model_ptr->switch_data=0x20;  //channel06
				}
				else if(slave_addr==54)  //0x36
				{
					model_ptr->switch_data=0x40;  //channel07
				}
				else if(slave_addr==55)  //0x37
				{
					model_ptr->switch_data=0x80;  //channel08
				}
				else
				{
				    	if (v) {
						fprintf(stderr, "8895 wrong Slave Address\n");
					}
					return -1;
				}
			}
			
			
			
			
			
									
			else
			{
                
                if (v) {                      
				       fprintf(stderr, "Fail to Set Switch Date\n");
                }
                return -1;
								
			}
			//model_ptr->switch_data=0xff;
			if(0)
			{
				fprintf(stdout, "model name : %s\n",model_ptr->model);
				fprintf(stdout, "model addr : %x\n",model_ptr->switch_addr);
				fprintf(stdout, "model data : %x\n",model_ptr->switch_data);
				fprintf(stdout, "addr : %x\n",slave_addr);
			}
			if(i2c_smbus_write_byte(fd, model_ptr->switch_addr, model_ptr->switch_data, 0, NULL)) {
				if (v)
                fprintf(stderr, "Get through I2C BUS FAILED\n");
				return -1;
			}
                if (v)
                fprintf(stdout, "Get through I2C Bus successful\n");
			return 0;
		}
		model_ptr++;
    }
    return -1;
}
 
// i2c write byte
int i2c_smbus_write_byte(int fd, unsigned char i2c_address,
                        unsigned char i2c_command,
                        int pec, unsigned char *pec_byte)
{
	bp_s bp;

	bp.bp_addr = i2c_address;
	bp.bp_cmd = i2c_command;
	
    if(pec_byte!=NULL)
	{
		bp.bp_pec = *pec_byte;
	}
	else
	{
		bp.bp_pec = i2c_command;
	}
	//bp.bp_pec = *pec_byte;
	memset(bp.bp_buffer, 0, sizeof(bp.bp_buffer));

	return ( ioctl(fd, IOCTL_WRITE_BYTE_CMD, &bp));
}

// i2c write byte data
int i2c_smbus_write_byte_data(int fd, unsigned char i2c_address, 
                                unsigned char i2c_command, unsigned char value)
{

	bp_s bp;

	bp.bp_addr = i2c_address;
	bp.bp_cmd = i2c_command;
	bp.bp_pec = 0;
	memset(bp.bp_buffer, 0, sizeof(bp.bp_buffer));
	bp.bp_buffer[0] = value;

	return ( ioctl(fd, IOCTL_WRITE_BYTE_DATA_CMD, &bp));
}

// i2c read byte
int i2c_smbus_read_byte_data(int fd, unsigned char i2c_address,
                             unsigned char i2c_command, unsigned char *ret_val, int inhibit_err)
{

	bp_s 	bp;
	int	ret;

	bp.bp_addr = i2c_address;
	bp.bp_cmd = i2c_command;
	bp.bp_pec = 0;
	bp.bp_inhibit_err = inhibit_err;
	memset(bp.bp_buffer, 0, sizeof(bp.bp_buffer));
	
	ret =  ioctl(fd, IOCTL_READ_BYTE_DATA_CMD, &bp);
	*ret_val = bp.bp_buffer[0];
	return ret;
}

// i2c read word
int i2c_smbus_read_word_data(int fd, unsigned char i2c_address,
                            unsigned char i2c_command, unsigned short *ret_val)
{

	bp_s 	bp;
	int	ret;
	unsigned short tmp;

	bp.bp_addr = i2c_address;
	bp.bp_cmd = i2c_command;
	bp.bp_pec = 0;
	memset(bp.bp_buffer, 0, sizeof(bp.bp_buffer));
	
	ret =  ioctl(fd, IOCTL_READ_WORD_DATA_CMD, &bp);
	tmp = bp.bp_buffer[1];
	tmp <<= 8;
	tmp |= bp.bp_buffer[0];
	
    *ret_val = tmp;
	
	return ret;

}

