/*-
 * Copyright (c) 2014, ServerU Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
 
#include "bp_def.h"

/*
#ifndef DJGPP
#define inportb(x) inb(x)
#define outportb(x, y) outb(y, x)
#define inportl(x) inl(x)
#define outportl(x, y) outl(y, x)
#endif
*/

/* SMBUS timeout setting */
#define MAX_TIMEOUT             1000

/* Intel 801 SMBus address offsets */
#define SMBHSTSTS       (0)
#define SMBHSTCNT       (2)
#define SMBHSTCMD       (3)
#define SMBHSTADD       (4)
#define SMBHSTDAT0      (5)
#define SMBHSTDAT1      (6)
#define SMBBLKDAT       (7)
#define SMBPEC          (8)         /* ICH3 and later */
#define SMBAUXSTS       (12)        /* ICH4 and later */
#define SMBAUXCTL       (13)        /* ICH4 and later */

/* I801 Hosts Status register bits */
#define SMBHSTSTS_BYTE_DONE     0x80
#define SMBHSTSTS_INUSE_STS     0x40
#define SMBHSTSTS_SMBALERT_STS  0x20
#define SMBHSTSTS_FAILED        0x10
#define SMBHSTSTS_BUS_ERR       0x08
#define SMBHSTSTS_DEV_ERR       0x04
#define SMBHSTSTS_INTR          0x02
#define SMBHSTSTS_HOST_BUSY     0x01

/* kill bit for SMBHSTCNT */
#define SMBHSTCNT_KILL          2

/* SMBUS ARP assigned base address */
#define ARP_BASE_ADDR   0x20

/* SMBUS 2.0 ARP UUID definition */
#define ARP_UUID_VID0   2   //Vendor ID
#define ARP_UUID_VID1   3   //Vendor ID
#define ARP_UUID_VSID0  12  //Vendor Specific ID

/*
typedef struct {
	unsigned char bp_addr;
	unsigned char bp_cmd;
	unsigned char bp_pec;
	unsigned char bp_inhibit_err;
	unsigned char bp_buffer[20];
} bp_s;
*/

#define I2C_READ        0x01
#define I2C_WRITE       0x00
/* return 0 mean host ready for transfer, otherwise return -1 */
int host_not_ready(int fd);

/* wait command processed and check error occured */
int host_busy_or_err(int fd, int inhibit_err);
int i2c_smbus_write_byte(int fd, unsigned char i2c_address,
                        unsigned char i2c_command,
                        int pec, unsigned char *pec_byte);
int i2c_smbus_write_byte_data(int fd, unsigned char i2c_address,
                                unsigned char i2c_register, unsigned char value);

int i2c_smbus_read_byte_data(int fd, unsigned char i2c_address,
                             unsigned char i2c_register, unsigned char *ret_val, int inhibit_err);

int i2c_smbus_read_word_data(int fd, unsigned char i2c_address,
                            unsigned char i2c_register, unsigned short *ret_val);

int i2c_smbus_read_block_data(int fd, unsigned char i2c_address,
                            unsigned char i2c_register, unsigned char ret_array[],
                            unsigned char *pec_byte);

int i2c_smbus_write_block_data(int fd, unsigned char i2c_address,
                            unsigned char i2c_register, unsigned char ret_array[],
                            unsigned char *pec_byte);


